#ifndef MAINWINDOW_H 
#define MAINWINDOW_H

#include <QMainWindow>

#include "branchpropwindow.h"
#include "file.h"
#include "findwindow.h"
#include "historywindow.h"
#include "mapeditor.h"
#include "texteditor.h"
#include "xml.h"

enum ModMode {ModModeNone,ModModeColor,ModModeCopy,ModModeXLink};

class Main : public QMainWindow 
{
    Q_OBJECT

public:
    Main(QWidget* parent=0, const char* name=0, Qt::WFlags f=0);
    ~Main();
	void loadCmdLine();
	void statusMessage (const QString &);

public slots:
    void fileNew();

protected:
	void closeEvent( QCloseEvent* );

private:
    void setupFileActions();
    void setupEditActions();
    void setupFormatActions();
    void setupViewActions();
    void setupModeActions();
    void setupWindowActions();
    void setupFlagActions();
    void setupSettingsActions();
    void setupTestActions();
    void setupHelpActions();
    void setupContextMenus();
	void setupRecentMapsMenu();
	void hideEvent (QHideEvent * );
	void showEvent (QShowEvent * );
	bool reallyWriteDirectory(const QString&);
	QString browseDirectory(const QString&);
	MapEditor* currentMapEditor() const;
    
private slots:
	void editorChanged(QWidget*);

    ErrorCode fileLoad(QString ,const LoadMode &);
    void fileLoad(const LoadMode &);
    void fileLoad();
	void fileLoadRecent();
	void addRecentMap (const QString &);
    void fileSave(const SaveMode & );
    void fileSave();
    void fileSaveAs(const SaveMode &);
    void fileSaveAs();
    void fileImportKDEBookmarks();
    void fileImportFirefoxBookmarks();
    void fileImportMM();
    void fileImportDir();
    void fileExportXML();
    void fileExportXHTML();
    void fileExportImage();
    void fileExportASCII();
    void fileExportLaTeX();
    void fileExportKDEBookmarks();
    void fileExportTaskjuggler();
    void fileExportOOPresentation();
    void fileCloseMap();
    void filePrint();
    void fileExitVYM();

public slots:
    void editUndo();	
    void editRedo();	
	void gotoHistoryStep (int);
private slots:	
    void editCopy();	
    void editPaste();	
    void editCut();	
    void editOpenFindWindow();
	void editFind(QString);
	void editFindChanged();
private:
	void openTabs(QStringList);
public slots:
	void editOpenURL();
	void editOpenURLTab();
private slots:
	void editOpenMultipleURLTabs();
	void editURL();
	void editHeading2URL();
	void editBugzilla2URL();
	void editFATE2URL();
	void openVymLinks(const QStringList &);
	void editVymLink();
	void editOpenMultipleVymLinks();
    void editHeadingFinished();
public slots:
    void editHeading();
	void editOpenVymLink();
private slots:
	void editDeleteVymLink();
	void editToggleHideExport();
	void editMapInfo();
    void editMoveUp();	
    void editMoveDown();	
    void editToggleScroll();
    void editUnScrollAll();
    void editNewBranch();
    void editNewBranchBefore();
    void editNewBranchAbove();
    void editNewBranchBelow();
    void editImportAdd();
    void editImportReplace();
    void editSaveBranch();
    void editDeleteKeepChilds();
    void editDeleteChilds();
    void editDeleteSelection();
    void editUpperBranch();
    void editLowerBranch();
    void editLeftBranch();
    void editRightBranch();
    void editFirstBranch();
    void editLastBranch();
    void editLoadImage();
    void editSaveImage();
    void editFollowXLink (QAction *);
    void editEditXLink (QAction *);

    void formatSelectColor();
    void formatPickColor();
    void colorChanged(QColor);
    void formatColorBranch();
    void formatColorSubtree();
	void formatLinkStyleLine();
	void formatLinkStyleParabel();
	void formatLinkStylePolyLine();
	void formatLinkStylePolyParabel();
    void formatSelectBackColor();
    void formatSelectBackImage();
    void formatSelectLinkColor();
    void formatToggleLinkColorHint();
	void formatIncludeImagesVer();
	void formatIncludeImagesHor();
	void formatHideLinkUnselected();

    void viewZoomReset();
    void viewZoomIn();
    void viewZoomOut();

public slots:
	bool settingsPDF();
	bool settingsURL();
	void settingsToggleDelKey();

	void windowToggleNoteEditor();
	void windowToggleHistory();
	void updateHistory(SimpleSettings &);
	void windowToggleAntiAlias();
	void windowToggleSmoothPixmap();
	void updateNoteFlag();
	void updateActions();
	ModMode getModMode();
	bool autoEdit();
	bool autoSelectHeading();
	bool useFlagGroups();

private slots:
	void showPropertyDialog();
	void windowNextEditor();
	void windowPreviousEditor();
	void windowShowNoteEditor();
	void windowHideNoteEditor();

	void standardFlagChanged();

    void testFunction();
    void testCommand();

    void helpDoc();
    void helpAbout();
    void helpAboutQT();

private:
	QTabWidget *tabWidget;
	FindWindow *findWindow;
	QProcess *procBrowser;

	QStringList imageTypes;

	QLineEdit *lineedit;	// to enter headings of branches
	QString prevSelection;

	HistoryWindow *historyWindow;

	BranchPropertyWindow *branchPropertyWindow;

	QList <QAction*> actionListBranches;

	QColor currentColor;

	QMenu *recentFilesMenu;
	enum { MaxRecentFiles = 9 };
    QAction *recentFileActs[MaxRecentFiles];

	QAction* actionFileSave;
	QAction* actionFilePrint;
	QAction* actionEditUndo;
	QAction* actionEditRedo;
	QAction *actionEditCopy;
	QAction *actionEditCut;
	QAction *actionEditPaste;
	QAction *actionEditMoveUp;
	QAction *actionEditMoveDown;
	QAction *actionEditToggleScroll;
	QAction* actionEditOpenURL;
	QAction* actionEditOpenURLTab;
	QAction* actionEditOpenMultipleURLTabs;
	QAction* actionEditURL;
	QAction* actionEditHeading2URL;
	QAction* actionEditBugzilla2URL;
	QAction* actionEditFATE2URL;
	QAction *actionEditOpenVymLink;
	QAction *actionEditOpenMultipleVymLinks;
	QAction *actionEditVymLink;
	QAction *actionEditDeleteVymLink;
	QAction *actionEditToggleHideExport;
	QAction *actionEditMapInfo;
	QAction *actionEditHeading;
	QAction *actionEditDelete;
	QAction *actionEditAddBranch;
	QAction *actionEditAddBranchBefore;
	QAction *actionEditAddBranchAbove;
	QAction *actionEditAddBranchBelow;
	QAction *actionEditDeleteKeepChilds;
	QAction *actionEditDeleteChilds;
	QAction *actionEditImportAdd;
	QAction *actionEditImportReplace;
	QAction *actionEditSaveBranch;
	QAction *actionEditSelectFirst;
	QAction *actionEditSelectLast;
	QAction *actionEditLoadImage;

	QAction* actionFormatColor;
	QAction* actionFormatPickColor;
	QAction* actionFormatColorBranch;
	QAction* actionFormatColorSubtree;
	QAction* actionFormatLinkColorHint;
	QAction* actionFormatBackColor;
	QAction* actionFormatBackImage;
	QAction* actionFormatLinkColor;
	QAction *actionFormatIncludeImagesVer;
	QAction *actionFormatIncludeImagesHor;

	QActionGroup* actionGroupModModes;
	QAction* actionModModeColor;
	QAction* actionModModeXLink;
	QAction* actionModModeCopy;

	QActionGroup *actionGroupFormatFrameTypes;

	QAction *actionOpenPropertyDialog;

	QActionGroup *actionGroupFormatLinkStyles;
	QAction *actionFormatLinkStyleLine;
	QAction *actionFormatLinkStyleParabel;
	QAction *actionFormatLinkStylePolyLine;
	QAction *actionFormatLinkStylePolyParabel;
	QAction *actionFormatHideLinkUnselected;

	QAction *actionViewToggleNoteEditor;
	QAction *actionViewToggleHistoryWindow;
	QAction *actionViewToggleAntiAlias;
	QAction *actionViewToggleSmoothPixmapTransform;

	QAction* actionSettingsAutoEdit;
	QAction* actionSettingsAutoSelectHeading;
	QAction* actionSettingsAutoSelectText;
	QAction* actionSettingsUseDelKey;
	QAction* actionSettingsUseFlagGroups;
	QAction* actionSettingsUseHideExport;
};


#endif

